/*

  I build this with:

  `gcc -o test test.cpp -lkaffevm -I/usr/local/include/kaffe`

  Then run:

  `test 0 0`     => SUCCESS
  `test 0 100`   => SUCCESS
  `test 100 100` => SUCCESS
  `test 100 0`   => FAILURE
  `test 100 70`  => FAILURE

  etc.

  If you have a good fix for this, please email me:

  ketrenoj@austin.rr.com

  Thanks,
  James
  
 */


#include <stdlib.h>
#include <string.h>
#include <jni.h>

JavaVM* m_JavaVM = NULL;
JNIEnv* m_JNIEnv = NULL;
JavaVMInitArgs vm_args;

class CTest
{
public:
    int NestInit(int i)
        {
            char test[100];
            if (i == 0)
                return InitializeJVM();
            return NestInit(i-1);
        }

    int NestUse(int i)
        {
            char test[100];
            if (i == 0)
                return UseJVM();
            return NestUse(i-1);
        }
    
    int InitializeJVM()
        {
            memset(&vm_args, '\0', sizeof(vm_args));
            vm_args.version = 0x00010001;

            JNI_GetDefaultJavaVMInitArgs(&vm_args);

            vm_args.classpath = "/usr/local/share/kaffe/Klasses.jar";
            vm_args.libraryhome = "/usr/local/lib/kaffe";
            vm_args.classhome = "/usr/local/share/kaffe";

            if (JNI_CreateJavaVM(&m_JavaVM,
                                     &m_JNIEnv,
                                     &vm_args) < 0)
            {
                printf("0x%08lX - ...can't create Java VM.\n",
                        (long)this);
                return 0;
            }

            printf("0x%08lX - ...returning JavaVM = 0x%08lX, "
                   "JNIEnv = 0x%08lX\n",
                   (long)this, (long)m_JavaVM, (long)m_JNIEnv);

            return 1;
        }

    int UseJVM()
        {

            jclass java_lang_String = m_JNIEnv->FindClass("java/lang/String");
            if (java_lang_String)
            {
                printf("0x%08lX - ...java/lang/String found.\n",
                        (long)this);
                return 1;
            }

            printf("0x%08lX - ...java/lang/String NOT found.\n",
                    (long)this);

            return 0;
        }
};

void main(int argc, char* argv[])
{
    CTest Test;
    int Init = 0;
    int Use = 0;
    
    if (argc == 3)
    {
        Use = atoi(argv[2]);
        Init = atoi(argv[1]);
    }

    if (argc == 2)
        Init = atoi(argv[1]);

    printf("> This will output saying it is returning the JavaVM.\n");
    printf("> The call will be nested %d times with 100 bytes of local\n",
           Init);
    printf("> storage per nest.\n");
    printf("> \n> Calling NestInit(%d)...\n\n", Init);

    Test.NestInit(Init);

    printf("\n> NestInit() Returned.\n>\n");

    printf("> Now attempting to use the returned JavaVM.\n");
    printf("> This call will be nested %d times, also with 100 bytes of\n",
           Use);
    printf("> local storage per nest.  If it doesn't say it found\n");
    printf("> java/lang/String then things are broken.  Providing a\n");
    printf("> 'Use' nest where:\n");
    printf("> Init - Use >= ~30 (ie., `test 1000 970`) will result in the\n");
    printf("> failure.\n");
    printf("> \n> Calling NestUse(%d)...\n\n", Use);
    
    Test.NestUse(Use);

    printf("\n> NestUse() Returned.\n> \n");

    printf("> Press ENTER to exit.\n");
    getchar();
}
